@@ -15,3 +15,9 @@ pub mod node_list;
 pub mod string_map;
 pub mod location;
 pub mod storage;
+pub mod blob;
+pub mod file;
+pub mod file_list;
+pub mod file_reader;
+pub mod array_buffer;
+pub mod typed_array;
